import os

basedir = os.path.abspath(os.path.dirname(__file__))


class Config:
    KNOWN_MANUAL_INPUTS = [
        "entrada manual",
        "entrada digitada",
        "entrada do usuario",
        "input manual",
        "user input",
        "Input do usuario",
        "input_inesperado",
        "Outro",
        "Outros",
        "recomecar",
        "reiniciar",
        "voltar ao inicio",
        "NaN",
        "None",
        "N/A"
    ]
    
    SCALE_TRANSLATIONS = [
        "Pessimo",
        "Terrivel",
        "Vergonhoso",
        ":tired_face:",
        ":crying_face:",
        "Sad",
        "Muito mal",
        "Muito ruim",
        "Ruim",
        "nao",
        "Nao gostei",
        "Nao curti",
        "Nao resolveu",
        "Nao ajudou",
        "Nao ajuda",
        "Nao adiantou",
        "Nao deu certo",
        "Nao permitiu",
        "Nao satisfez",
        "Insatisfatorio",
        "Nao viabilizou",
        "Nao auxiliou",
        "Nao totalmente",
        "Muito Dificil",
        "Dificil",
        "Fraco",
        "confuso",
        "Demorado",
        "lento",
        "Pode melhorar",
        ":slightly_frowning_face:",
        "Nao sei",
        "Em duvida",
        "Indiferente",
        "Medium",
        "Mais ou menos",
        "Ok",
        "Okay",
        "Na media",
        "Nao quero responder",
        ":neutral_face:",
        "Neutro",
        "Permitiu",
        "Facil",
        "Muito Facil",
        "Viabilizou",
        "Auxiliou",
        "Adiantou",
        "Regular",
        "Satisfatorio",
        "Satisfeito",
        "Resolveu",
        "Favoreceu",
        "Ajuda",
        "Ajudou",
        "Ajuda sim",
        "Ajuda_sim",
        "Ajudou sim",
        "Ajudou_sim",
        "Sim, ajudou",
        "Sim, ajuda",
        "Bom",
        "Legal",
        "Apreciei",
        "Gostei",
        "Happy",
        ":slightly_smiling_face:",
        "Muito bem",
        "muito_bem",
        "Muito bom",
        "muito_bom",
        "Gostei muito",
        "Otimo",
        "Apreciei",
        "Adorei",
        "Amei",
        ":smiling_face_with_heart-eyes:",
        "Excelente"
    ]



"""Core of the satisfaction rate metric.

This part of the process use a numeric scale.
Therefore, if the original input was a textual scale, it is necessary to use this metric to transform to numeric scale.

Methods:
____________
    * calculate_satisfaction_rate - Calculates the satisfaction rate based on the bot scale.
"""

__all__ = [
    "calculate_satisfaction_rate"
]

from .calculate_satisfaction_rate import calculate_satisfaction_rate



import unittest
import pandas as pd

import take_satisfaction as ts


class TestRun(unittest.TestCase):
    bot_identity = "centaurobc@msging.net"
    start_date = "2020-06-01"
    end_date = "	2020-07-01"
    
    pdf = pd.DataFrame(
        {"Action": ["Pessimo", "Excelente", "Otimo", "Ok", "Ruim"],
         "amount": [173, 57, 114, 168, 58]})
    
    result = ts.run(dataframe=pdf,
                    scale_column="Action",
                    amount_column="amount",
                    similarity_threshold=80)
    
    def test_run_rate(self):
        """Check if rate is calculated as expected."""
        expected = 0.42280701754385963
        actual = self.result["rate"]
        self.assertEqual(expected, actual)

    def test_run_default_similarity_threshold(self):
        """Check if rate is calculated as expected."""
        expected = 80
        actual = self.result["similarity_threshold"]
        self.assertEqual(expected, actual)
        
    def test_return_type(self):
        """Check if return is type dict"""
        self.assertIsInstance(self.result, dict)
    
    def test_return_struct(self):
        """Check if return has default structure"""
        expected = ["rate", "similarity_threshold", "operation"]
        actual = list(self.result.keys())
        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



import pandas as pd


DF = pd.DataFrame


def input_type_validation(dataframe: DF, similarity_threshold: int) -> None:
    """Validate run input type.
    
    :param dataframe: A dataframe containing `CSR` scales and the `amount` per scale.
    :type: ``pandas.DataFrame``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    :raise TypeError: if any input is not instance of expected type.
    """
    if not isinstance(dataframe, DF):
        raise TypeError("`dataframe` input must be type `pandas.DataFrame` type")
    
    if not isinstance(similarity_threshold, int):
        raise ValueError("`similarity_threshold` input must be type `int`")


import pandas as pd

from .df_validation import dataframe_validation
from .run_input_validation import input_type_validation


def data_validation(dataframe: pd.DataFrame,
                    scale_column: str,
                    amount_column: str,
                    similarity_threshold: int = 83) -> None:
    """
    
    :param dataframe: A dataframe containing `CSR` scales and the `amount` per scale.
    :type: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type: ``str``
    :param amount_column: Name of column with amount per scale.
    :type: ``str``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    """
    input_type_validation(dataframe=dataframe,
                          similarity_threshold=similarity_threshold)
    
    dataframe_validation(df=dataframe,
                         scale_column=scale_column,
                         amount_column=amount_column)



import typing as tp
import pandas as pd

from satisfaction_config import Config
from take_satisfaction.preprocess import remove_manual, convert_scale
from take_satisfaction.core import calculate_satisfaction_rate
from take_satisfaction.data_validation import dataframe_validation, data_validation


def run(dataframe: pd.DataFrame, scale_column: str, amount_column: str, similarity_threshold: int = 83) -> tp.Dict[str, tp.Any]:
    """Run Take Satisfaction

    :param dataframe: A dataframe containing `CSR` scales and the `amount` per scale.
    :type: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type: ``str``
    :param amount_column: Name of column with amount per scale.
    :type: ``str``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    :return: Satisfaction rate in report mode.
    :rtype:  ``typing.Dict[str, typing.Any]``
    """
    data_validation(dataframe=dataframe,
                    scale_column=scale_column,
                    amount_column=amount_column,
                    similarity_threshold=similarity_threshold)
    
    preprocessed_df = remove_manual(df=dataframe,
                                    known_manual_inputs=Config.KNOWN_MANUAL_INPUTS,
                                    similarity_threshold=similarity_threshold,
                                    scale_column=scale_column)
    
    dataframe_validation(df=preprocessed_df,
                         scale_column=scale_column,
                         amount_column=amount_column)
    
    converted_df = convert_scale(df=preprocessed_df,
                                 scale_column=scale_column,
                                 scale_translations=Config.SCALE_TRANSLATIONS,
                                 similarity_threshold=similarity_threshold)
    
    satisfaction_rate = calculate_satisfaction_rate(df=converted_df,
                                                    scale_column=scale_column,
                                                    amount_column=amount_column)
    
    return {
        "rate": satisfaction_rate,
        "similarity_threshold": similarity_threshold,
        "operation": {
            "input": dataframe,
            "preprocessed": preprocessed_df,
            "converted": converted_df
            }
        }



"""This project calculates the satisfaction rate for BLiP smart contacts.

It should be passed the full scale and amount per scale for the satisfaction rate core.

Methods:
_________
    * run: Run Take Satisfaction for given dataframe.
"""

__author__ = "squad XD"
__version__ = "1.0.0"

from .run import run



import unittest

from satisfaction_config import Config


class TestConfig(unittest.TestCase):
    def test_KNOWN_MANUAL_INPUTS(self):
        """Test if SATISFACTION_WEIGHT can be called from satisfaction_config and if has default value."""
        self.assertIsInstance(Config.KNOWN_MANUAL_INPUTS, list)
    
    def test_SCALE_TRANSLATIONS(self):
        """Test if RESOLUTION_WEIGHT can be called from satisfaction_config and if has default value."""
        self.assertIsInstance(Config.SCALE_TRANSLATIONS, list)


if __name__ == '__main__':
    unittest.main()



import typing as tp

import unidecode as uni
import emoji
import pandas as pd
from fuzzywuzzy import fuzz

from take_satisfaction.custom_exceptions import NoSuitableTranslationError

DF = pd.DataFrame
SR = pd.Series


def __convert_numeric_answer_column(df: DF, scale_column: str) -> DF:
    """Converts the target column's answers to integers by casting.

    :param df: A dataframe containing `CSR` scales and the `amount` per scale.
    :type df: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type scale_column: ``str``
    :return: Dataframe with the target column's answers converted into integers.
    :rtype: ``pandas.Dataframe``
    """
    df.loc[:, scale_column] = pd.to_numeric(df[scale_column])
    return df


def __convert_answer(answer: str, scale_translations: tp.List[str], similarity_threshold: int) -> int:
    """Converts answer to the index of its best match on the reference scale.

    Answer is like "Gostei", "Muito ruim"...
    And index range from negative to positive as they appear on the reference.
    Raises an exception if there is no reference similar enough.

    :param answer: String with scale answer.
    :type answer: ``str``
    :param scale_translations: List with answer variations appearing from negative to positive.
    :type scale_translations: ``list`` from ``str``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    :return: Index of the best match on the reference scale.
    :rtype: ``int``
    """
    answer = emoji.demojize(answer)
    similarity = [
        (fuzz.ratio(uni.unidecode(answer.lower()), uni.unidecode(value.lower())), idx)
        for idx, value in enumerate(scale_translations)
    ]
    
    max_similarity, position = max(similarity)
    
    if max_similarity >= similarity_threshold:
        return position
    raise NoSuitableTranslationError('Scale not found for: `' + str(answer) + '`')


def __convert_scale_to_indexes(df: DF,
                               scale_column: str,
                               scale_translations: tp.List[str],
                               similarity_threshold: int) -> DF:
    """Converts df column answers to the index of its best match on the reference scale.

    Raises an exception if the scale_conversion fails.

    :param df: A dataframe containing `CSR` scales and the `amount` per scale.
    :type df: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type scale_column: ``str``
    :param scale_translations: List with answer variations appearing from negative to positive.
    :type scale_translations: ``list`` from ``str``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    :return: Dataframe with answer column converted to satisfaction level.
    :rtype: ``pandas.Dataframe`
    """
    df_converted = df.copy()
    df_converted.loc[:, scale_column] = df[scale_column].apply(__convert_answer,
                                                               args=(scale_translations, similarity_threshold))
    return df_converted


def __normalizes_scale_answers(df: DF, scale_column: str) -> DF:
    """Normalizes df column answers to an integer scale with a step of one.

    :param df: A dataframe containing `CSR` scales and the `amount` per scale.
    :type df: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type scale_column: ``str``
    :return: Dataframe with answer column converted to satisfaction level.
    :rtype: ``pandas.Dataframe`
    """
    sorted_df = df.sort_values(by=scale_column, axis=0)
    sorted_df.loc[:, scale_column] = [i for i in range(len(sorted_df))]
    return sorted_df


def is_numeric_scale(df: DF, column: str) -> bool:
    """Validates whether the answers in the target column of the df are numeric.

    Returns True if the column's items are numeric.

    :param df: A dataframe containing `CSR` scales and the `amount` per scale.
    :type df: ``pandas.Dataframe``
    :param column: Column to assert data is numeric type.
    :type column: ``str``
    :return: Boolean indicating whether the column is numeric or not.
    :rtype: ``bool``
    """
    return pd.to_numeric(df[column], errors='coerce').notnull().all()


def convert_scale(df: DF, scale_column: str, scale_translations: tp.List[str], similarity_threshold: int) -> DF:
    """Converts df column answers to its satisfaction levels.

    :param df: A dataframe containing `CSR` scales and the `amount` per scale.
    :type: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type: ``str``
    :param scale_translations: List with answer variations appearing from negative to positive.
    :type scale_translations: ``list`` from ``str``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    :return: Dataframe with answer column converted to satisfaction level.
    :rtype: ``pandas.Dataframe``
    """
    if is_numeric_scale(df, column=scale_column):
        return __convert_numeric_answer_column(df=df, scale_column=scale_column) \
            .sort_values(by=scale_column, axis=0) \
            .reset_index(drop=True)

    converted_scale = __convert_scale_to_indexes(df,
                                                 scale_column=scale_column,
                                                 scale_translations=scale_translations,
                                                 similarity_threshold=similarity_threshold)
    return __normalizes_scale_answers(df=converted_scale,
                                      scale_column=scale_column)



"""Data Validation module.

The main function on this method is `data_validation`, called on the beginning of run code.
Every other validation method is for between process.

Methods:
____________
    * data_validation - Main function of validation layer.
    * dataframe_validation - Perform a content and structure validation.
"""


__all__ = ["data_validation",
           "dataframe_validation"]

from .main import data_validation
from .df_validation import dataframe_validation



from setuptools import find_packages, setup
from take_satisfaction import __author__ as author
from take_satisfaction import __version__ as version


short_description = "Generate a rate between 0 and 1 for Consumer Satisfaction Survey."
long_description = open('README.md').read()

install_requires = [
    req
    for req in [
        line.split('#', 1)[0].strip()
        for line in open('requirements.txt', 'r', encoding='utf-8')
    ]
    if req and not req.startswith('--')
]
classifiers = [
    'Programming Language :: Python :: 3.7',
    'Operating System :: OS Independent',
]

setup(
    name='take_satisfaction',
    keywords=['BLiP', 'score', 'satisfaction'],
    version=version,
    author=author,
    author_email='anaytics.ped@take.net',
    maintainer='Take - D&A',
    maintainer_email='anaytics.ped@take.net',
    description=short_description,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=['tests']),
    install_requires=install_requires,
    classifiers=classifiers,
    license="MIT License",
    python_requires='>=3.6'
)



import typing as tp

import numpy as np
import pandas as pd

NUMBER = tp.Union[float, int]
ARRAY = tp.Union[pd.Series, np.ndarray]
DF = pd.DataFrame
S = pd.Series


def __weighted_mean(x: ARRAY, w: ARRAY) -> float:
    """Takes weighted mean.

    :param x: Values to be averaged.
    :type x: ``pd.Series`` or ``np.ndarray``
    :param w: Weights to be applied.
    :type w: ``pd.Series`` or ``np.ndarray``
    :return: Weighted mean.
    :rtype: ``float``
    """
    return np.vdot(x, w) / np.sum(w)


def __scale(x: NUMBER, min_: NUMBER, max_: NUMBER) -> NUMBER:
    """Transform a value x to a scale between min_ and max_.

    :param x: Value to be converted.
    :type x: ``float`` or ``int``
    :param min_: Minimum value on scale.
    :type min_: ``float`` or ``int``
    :param max_: Maximum value on scale.
    :type max_: ``float`` or ``int``
    :return: Scaled value.
    :rtype: ``float`` or ``int``
    """
    return (x - min_) / (max_ - min_)


def calculate_satisfaction_rate(df: DF, scale_column: str, amount_column: str) -> float:
    """ Calculates the satisfaction rate based on the bot scale.

    :param df: Treated dataframe containing `CSR` scales as numeric and the `amount` per scale.
    :type df: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type scale_column: ``str``
    :param amount_column: Name of column with amount per scale.
    :type amount_column: ``str``
    :return: Satisfaction rate.
    :rtype: ``float``
    """
    mean = __weighted_mean(df[scale_column], df[amount_column])
    return __scale(mean, min(df[scale_column]), max(df[scale_column]))



"""Custom exceptions class for take_satisfaction package.

All classes inherit from `Exception` class.

Classes:
____________
    * InvalidDataframeError
    * AbsentColumnError
    * NoSuitableTranslationError
"""

__all__ = [
    "InvalidDataframeError",
    "AbsentColumnError",
    "NoSuitableTranslationError"
]

from .custom_exceptions import (InvalidDataframeError,
                                AbsentColumnError,
                                NoSuitableTranslationError)



import typing as tp

import numpy as np
import pandas as pd
from fuzzywuzzy import fuzz

DF = pd.DataFrame
S = pd.Series


def __match_manual(series: S, known_manual_inputs: tp.List[str], similarity_threshold: int) -> np.ndarray:
    """Returns a series describing whether each element in the series is a manual entry.
    
    :param series: The scales list to match.
    :type series: ``pandas.Series``
    :param known_manual_inputs: Manual entry variations.
    :type known_manual_inputs: ``list`` of ``str``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    :return: Series describing whether each element is a manual entry.
    :rtype: ``numpy.ndarray`` of ``bool``
    """
    return np.array([
        (isinstance(sample, str) and
         any(fuzz.ratio(sample.lower(), entry.lower()) >= similarity_threshold for entry in known_manual_inputs))
        for sample in series
    ])


def remove_manual(df: DF, known_manual_inputs: tp.List[str], similarity_threshold: int, scale_column: str) -> DF:
    """Checks for a manual entry row in the scale and removes it.

    :param df: Dataframe.
    :type df: ``pandas.DataFrame``
    :param known_manual_inputs: Manual entry variations.
    :type known_manual_inputs: ``list`` of ``str``
    :param similarity_threshold: Minimum similarity for str matching.
    :type similarity_threshold: ``int``
    :param scale_column: Target column to be analysed.
    :type scale_column: ``str``
    :return: Dataframe with the bot scale without manual entries.
    :rtype: ``pandas.DataFrame``
    """
    return df[~__match_manual(series=df[scale_column],
                              known_manual_inputs=known_manual_inputs,
                              similarity_threshold=similarity_threshold)]



"""Pre-process given input to create the necessary data to calculate the satisfaction rate.

Methods:
____________
    * remove_manual - Checks for a manual entry row in the scale and removes it.
    * convert_scale - Converts df column answers to its satisfaction levels.
"""

__all__ = ["remove_manual",
           "convert_scale"]

from .remove_manual import remove_manual
from .scale_conversion import convert_scale



import pandas as pd

from take_satisfaction.custom_exceptions import (InvalidDataframeError,
                                                 AbsentColumnError)

DF = pd.DataFrame


def dataframe_content_validation(df: DF) -> None:
    """Validates if df is not empty.
    
    :raise InvalidDataframeError: if dataframe is empty

    :param df: Dataframe with answers.
    :type df: ``pandas.Dataframe``
    """
    if df.empty:
        raise InvalidDataframeError('Cannot process empty scale')


def dataframe_has_columns(df: DF, scale_column: str, amount_column: str) -> None:
    """Validates if df is not empty, if it has an answer column and a quantity column.

    :raise AbsentColumnError: If `scale_column` or `amount_column` is not present on dataframe.
    
    :param df: Dataframe with satisfaction levels and its quantities.
    :type df: ``pandas.Dataframe``
    :param scale_column: Name of the scale column.
    :type: ``str``
    :param amount_column: Name of column with amount per scale.
    :type: ``str``
    """
    if scale_column not in df.columns:
        raise AbsentColumnError('Scale does not contain answer column `' + str(scale_column) + '`')
    
    if amount_column not in df.columns:
        raise AbsentColumnError('Scale does not contain quantity column `' + str(amount_column) + '`')
    
    
def dataframe_validation(df: DF, scale_column: str, amount_column: str) -> None:
    """Perform a content and structure validation.
    
    :param df: A dataframe containing `CSR` scales and the `amount` per scale.
    :type df: ``pandas.DataFrame``
    :param scale_column: Name of the scale column.
    :type: ``str``
    :param amount_column: Name of column with amount per scale.
    :type: ``str``
    """
    dataframe_has_columns(df=df,
                          scale_column=scale_column,
                          amount_column=amount_column)
    
    dataframe_content_validation(df=df)
